;; $Id: dblink.dsl 0.1 1997/07/31 14:51:33 nwalsh Exp $

;; ========================= LINKS AND ANCHORS ==========================

(element LINK 
  (let* ((target (element-with-id 
		  (attribute-string "LINKEND"))))
    (if (node-list-empty? target)
	(process-children)
	(make element gi: "A"
	      attributes: (list
			   (list "HREF" (href-to target)))
	      (process-children)))))

(element OLINK ($charseq$))

(element ULINK 
  (make element gi: "A"
	attributes: (list
		     (list "HREF" (attribute-string "URL")))
	(process-children)))

;; (element FOOTNOTEREF... in dbfootn.dsl)

(element XREF 
  (let* ((target (element-with-id 
		  (attribute-string "LINKEND" (current-node))))
	 (href (href-to target)))
    (make element gi: "A"
	  attributes: (list
		       (list "HREF" href))
	  (case (gi target)
	    (("STEP") (let ((para (node-list-first (select-elements
						    (descendants target)
						    "PARA"))))
			(with-mode inline-step-xref
			  (process-node-list para))))
	    (("BOOK" "SET" "APPENDIX" "CHAPTER" "EXAMPLE" "FIGURE" "PART"
		     "PREFACE" "REFSECT1" "REFSECT2" "REFSECT3" "SECT1"
		     "SECT2" "SECT3" "SECT4" "SECT5" "TABLE")
	     (let ((title (node-list-first (select-elements
					    (descendants target) "TITLE"))))
	       (with-mode inline-title-xref
		 (process-node-list title))))
	    (else (make sequence 
		    (literal "[XREF to " (gi target) " not supported]")))))))


(element ANCHOR (empty-sosofo))
(element BEGINPAGE (empty-sosofo))

(define (xref-part title)
  (make sequence
    (literal
     (gentext-xref-element-name-space (gi (parent title)))
     (format-number (ancestor-child-number "PART" title) "I")
     (gentext-xref-label-title-sep (gi (parent title))))
    (make sequence
      (process-children-trim))))

(define (xref-labeled-section title label)
  (let* ((sect (parent title))
	 (chn (ancestor-child-number "CHAPTER"))
	 (apn (ancestor-child-number "APPENDIX"))
	 (s1n (ancestor-child-number "SECT1"))
	 (s2n (ancestor-child-number "SECT2"))
	 (s3n (ancestor-child-number "SECT3"))
	 (s4n (ancestor-child-number "SECT4"))
	 (s5n (ancestor-child-number "SECT5"))
	 (isep (gentext-intra-label-sep (gi sect)))
	 (nsep (gentext-xref-label-title-sep (gi sect))))
    (make sequence
      (literal
       (gentext-xref-element-name-space (gi sect))
       (if (equal? label "AUTO")
	   (string-append
	    (cond
	     (chn (FNUM chn))
	     (apn (FNUM apn))
	     (else ""))
	    (if s1n (string-append isep (FNUM s1n)) "")
	    (if s2n (string-append isep (FNUM s2n)) "")
	    (if s3n (string-append isep (FNUM s3n)) "")
	    (if s4n (string-append isep (FNUM s4n)) "")
	    (if s5n (string-append isep (FNUM s5n)) "")
	    nsep)
	   (string-append label nsep)))
      (make sequence
	(process-children-trim)))))

(define (xref-unlabeled-section title)
  (let* ((sect (parent title))
	 (sectname (gentext-xref-element-name (gi sect)))
	 (chn (ancestor-child-number "CHAPTER"))
	 (apn (ancestor-child-number "APPENDIX")))
    (make sequence
      (literal
       (if (equal? sectname "")
	   ""
	   (string-append "the " (STRING-DOWNCASE sectname) " called ")))
      (make sequence
	(process-children-trim))
      (cond
       (chn (literal " in " (gentext-xref-element-name-space "CHAPTER") (FNUM chn)))
       (apn (literal " in " (gentext-xref-element-name-space "APPENDIX") (FNUM apn)))
       (else (literal ""))))))

(define (xref-section title)
  (let* ((sect (parent title))
	 (pname (gi sect))
	 (label-attr (attribute-string "LABEL" sect))
	 (label (if (not label-attr)
		    (if (and %section-autolabel% 
			     (not (equal? pname "SIMPLESECT")))
			"AUTO"
			"")
		    label-attr)))
    (if (equal? label "")
	(xref-unlabeled-section title)
	(xref-labeled-section title label))))

(define (xref-block title)
  (let* ((block (parent title))
	 (label (attribute-string "LABEL" (ancestor (gi block))))
	 (chn (ancestor-child-number "CHAPTER"))
	 (apn (ancestor-child-number "APPENDIX"))
	 (bkn (format-number (element-number (parent (current-node))) "1"))
	 (pnn (cond
	       (chn (FNUM chn))
	       (apn (FNUM apn))
	       (else "")))
	 (prefix (if (equal? pnn "") 
		     "" 
		     (string-append pnn 
				    (gentext-intra-label-sep (gi block))))))
    (make sequence
      (literal
       (string-append
	(gentext-xref-element-name-space (gi block))
	(if label
	    label
	    (string-append prefix bkn))
	(gentext-xref-label-title-sep (gi block))))
      (make sequence
	(process-children-trim)))))

(define (xref-preface title)
  (let* ((preface (parent title))
	 (label (attribute-string "LABEL" (ancestor (gi preface)))))
    (make sequence
      (literal
       (string-append
	(if label
	    (string-append 
	     (gentext-xref-element-name-space (gi preface))
	     label (gentext-xref-label-title-sep (gi preface)))
	    "")))
      (make sequence
	(process-children-trim)))))

(define (xref-chapapp title)
  (let* ((chapapp (parent title))
	 (label (attribute-string "LABEL" (ancestor (gi chapapp))))
	 (chn (ancestor-child-number "CHAPTER"))
	 (apn (ancestor-child-number "APPENDIX")))
    (make sequence
      (literal
       (string-append
	(gentext-xref-element-name-space (gi chapapp))
	(if label
	    label
	    (cond
	     (chn (FNUM chn))
	     (apn (FNUM apn))
	     (else ""))))
       (gentext-xref-label-title-sep (gi chapapp)))
      (make sequence
	(process-children-trim)))))

(mode inline-title-xref
  (element TITLE
    (let* ((pelem (parent (current-node))))
      (case (gi pelem)
	(("BOOK") (make sequence
		    (process-children-trim)))
	(("SET") (make sequence
		   (process-children-trim)))

	(("APPENDIX") (xref-chapapp (current-node)))
	(("CHAPTER") (xref-chapapp (current-node)))
	(("EXAMPLE") (xref-block (current-node)))
	(("FIGURE") (xref-block (current-node)))
	(("PART") (xref-part (current-node)))
	(("PREFACE") (xref-preface (current-node)))
	(("REFSECT1") (xref-section (current-node)))
	(("REFSECT2") (xref-section (current-node)))
	(("REFSECT3") (xref-section (current-node)))
	(("SECT1") (xref-section (current-node)))
	(("SECT2") (xref-section (current-node)))
	(("SECT3") (xref-section (current-node)))
	(("SECT4") (xref-section (current-node)))
	(("SECT5") (xref-section (current-node)))
	(("TABLE") (xref-block (current-node)))

	(else (make sequence 
		(literal "[XREF to " (gi pelem) " not supported]")))))))

(mode inline-step-xref
  (element (STEP PARA)
    (make sequence
      (literal
       (string-append
	(gentext-xref-element-name "STEP")
	" "
	($proc-step-xref-number$ (current-node)))))))
