;; $Id: dbsect.dsl 0.1 1997/07/31 14:51:33 nwalsh Exp $

;; ============================== SECTIONS ==============================

;; BRIDGEHEAD isn't a proper section, but appears to be a section title
(element BRIDGEHEAD ($lowtitle$ 2))

(define (SECTLEVEL)
  (if (have-ancestor? "SIMPLESECT")
      ;; SimpleSect is special, it should be level "n+1", where "n" is
      ;; the level of the numbered section that contains it.  If it is
      ;; the *first* sectioning element in a chapter, make it 
      ;; %default-simplesect-level%
      (cond
       ((have-ancestor? "SECT5") 6)
       ((have-ancestor? "SECT4") 5)
       ((have-ancestor? "SECT3") 4)
       ((have-ancestor? "SECT2") 3)
       ((have-ancestor? "SECT1") 2)
       (else %default-simplesect-level%))
      (cond
       ((have-ancestor? "SECT5") 5)
       ((have-ancestor? "SECT4") 4)
       ((have-ancestor? "SECT3") 3)
       ((have-ancestor? "SECT2") 2)
       ((have-ancestor? "SECT1") 1)
       ((have-ancestor? "REFERENCE") 1)
       (else 0)))) ;; 0 is the component (chapter/appendix) level

(define ($section$) ($block-container$))
(define ($sectitle$)
  (let* ((sect (parent (current-node)))
	 (renderas (inherited-attribute-string "renderas" sect))
	 (hlevel                         ;; the apparent section level;
	   (if renderas                  ;; if not real section level,
	     (string->number             ;;   then get the apparent level
	       (substring renderas 4 5)) ;;   from "renderas",
	     (SECTLEVEL)))               ;; else use the real level
	 (helem
	  (string-append "H" (number->string (+ hlevel 1))))
	 (label-attr (attribute-string "label" sect))
	 (label (if (not label-attr)
		    (if %section-autolabel%
			"AUTO"
			"")
		    label-attr))
	 (name (element-id))
	 (isep (gentext-intra-label-sep (gi sect)))
	 (nsep (gentext-label-title-sep (gi sect))))
    (make element gi: helem
	  (make element gi: "A"
		attributes: (list
			     (list "NAME" name))
		(literal
		 ;; SimpleSects are never AUTO numbered...not hierarchical
		 (if (and (not (equal? label ""))
			  (or (not (equal? label "AUTO"))
			      (not (equal? (gi sect) "SIMPLESECT"))))
		     (if (equal? label "AUTO")
			 (let* ((chn (ancestor-child-number "CHAPTER"))
				(apn (ancestor-child-number "APPENDIX"))
				(s1n (ancestor-child-number "SECT1"))
				(s2n (ancestor-child-number "SECT2"))
				(s3n (ancestor-child-number "SECT3"))
				(s4n (ancestor-child-number "SECT4"))
				(s5n (ancestor-child-number "SECT5"))
				(divnum (cond              ;; the chapter-level
					 (chn (FNUM chn))  ;; container number
					 (apn (FNUM apn))
					 (else ""))))
			   (string-append
			    divnum
			    (if s1n 
				(string-append
				 (if (not (equal? divnum ""))
				     isep
				     "")
				 (FNUM s1n)) 
				"")
			    (if s2n (string-append isep (FNUM s2n)) "")
			    (if s3n (string-append isep (FNUM s3n)) "")
			    (if s4n (string-append isep (FNUM s4n)) "")
			    (if s5n (string-append isep (FNUM s5n)) "")
			    nsep))
			 (string-append label nsep))
		     (string-append "")))
		(process-children-trim)))))

;;(element SECT1 ($section$))
(element (SECT1 TITLE) ($sectitle$))

(element SECT1
  (let ((chap-num (child-number (parent)))
	(sect-num (child-number (current-node))))
    (if (> sect-num 1)
	(make entity
	  system-id: (html-file)
	  (make document-type
	    name: "HTML"
	    public-id: %html-pubid%)
	  (make element gi: "HTML"
		(make element gi: "HEAD"
		      (make element gi: "TITLE"
			    (with-mode admon-caption-mode
				  (process-first-descendant "TITLE"))))
		(make element gi: "BODY" 
		      attributes: (list 
				   (list "BGCOLOR" "#FFFFFF")
				   (list "TEXT" "#000000"))
		      (header-navigation (current-node))
		      (process-children)
		      (footer-navigation (current-node)))))
	(process-children))))

(element SECT2 ($section$))
(element (SECT2 TITLE) ($sectitle$))
(element SECT3 ($section$))
(element (SECT3 TITLE) ($sectitle$))
(element SECT4 ($section$))
(element (SECT4 TITLE) ($sectitle$))
(element SECT5 ($section$))
(element (SECT5 TITLE) ($sectitle$))

(element SIMPLESECT ($section$))
(element (SIMPLESECT TITLE) ($sectitle$))

